package middleware

import (
	"context"
	"fmt"

	"git.perx.ru/perxis/perxis-go/id"
	"git.perx.ru/perxis/perxis-go/pkg/environments"

	logzap "git.perx.ru/perxis/perxis-go/zap"
	"go.uber.org/zap"
)

type loggingMiddleware struct {
	logger *zap.Logger
	next   environments.Environments
}

func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next environments.Environments) environments.Environments {
		return &loggingMiddleware{
			next:   next,
			logger: logger.With(logzap.Component("Environments")),
		}
	}
}

func (m *loggingMiddleware) Create(ctx context.Context, env *environments.Environment) (created *environments.Environment, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(environments.EventCreate),
	)

	created, err = m.next.Create(ctx, env)
	if err != nil {
		logger.Error("Failed to create", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog), logzap.Object(env))
		return
	}

	logger.Info("Environment created", logzap.Channels(logzap.Userlog), logzap.Object(created))

	return created, err
}

func (m *loggingMiddleware) Delete(ctx context.Context, spaceId string, envId string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(environments.EventDelete),
		logzap.Object(id.NewEnvironmentId(spaceId, envId)),
	)

	err = m.next.Delete(ctx, spaceId, envId)
	if err != nil {
		logger.Error("Failed to delete", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Environment deleted", logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) Get(ctx context.Context, spaceId string, envId string) (env *environments.Environment, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Object(id.NewEnvironmentId(spaceId, envId)),
	)

	env, err = m.next.Get(ctx, spaceId, envId)
	if err != nil {
		logger.Error("Failed to get", zap.Error(err))
		return
	}

	return env, err
}

func (m *loggingMiddleware) List(ctx context.Context, spaceId string) (envs []*environments.Environment, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	envs, err = m.next.List(ctx, spaceId)
	if err != nil {
		logger.Error("Failed to list", zap.Error(err))
		return
	}

	return envs, err
}

func (m *loggingMiddleware) Migrate(ctx context.Context, spaceId string, envId string, options ...*environments.MigrateOptions) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(environments.EventMigrate),
		logzap.Object(id.NewEnvironmentId(spaceId, envId)),
	)

	err = m.next.Migrate(ctx, spaceId, envId, options...)
	if err != nil {
		logger.Error("Failed to migrate", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	// Информация о миграции будет отправлена в сервис логирования внутри самого сервиса.

	return err
}

func (m *loggingMiddleware) RemoveAlias(ctx context.Context, spaceId string, envId string, alias string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(environments.EventUpdate),
		logzap.Object(id.NewEnvironmentId(spaceId, envId)),
	)

	err = m.next.RemoveAlias(ctx, spaceId, envId, alias)
	if err != nil {
		logger.Error(fmt.Sprintf("Failed to remove alias '%s'", alias), zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info(fmt.Sprintf("Environment alias '%s' removed", alias), logzap.Channels(logzap.Userlog))
	return err
}

func (m *loggingMiddleware) SetAlias(ctx context.Context, spaceId string, envId string, alias string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(environments.EventUpdate),
		logzap.Object(id.NewEnvironmentId(spaceId, envId)),
	)

	err = m.next.SetAlias(ctx, spaceId, envId, alias)
	if err != nil {
		logger.Error(fmt.Sprintf("Failed to set alias '%s'", alias), zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info(fmt.Sprintf("Environment alias '%s' set", alias), logzap.Channels(logzap.Userlog))
	return err
}

func (m *loggingMiddleware) Update(ctx context.Context, env *environments.Environment) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(environments.EventUpdate),
		logzap.Object(env),
	)

	err = m.next.Update(ctx, env)
	if err != nil {
		logger.Error("Failed to update", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Environment updated", logzap.Channels(logzap.Userlog))
	return err
}
